<?php

namespace App\Http\Controllers\Wechat\OfficialAccount;

use Illuminate\Routing\Controller as ChannelBaseController;
use App\Modules\OfficialAccount\Models\OfficialAccount;
use App\Modules\OfficialAccount\Models\OfficialInteractiveEvent;
use App\Modules\Channel\Models\Channel;
use App\Http\Controllers\Wechat\OfficialAccount\Transformers\OfficialMenuTransformer;
use Illuminate\Http\Request;
use GuzzleHttp\Client;
use App\Libs\OSS;

class OfficialMenusController extends ChannelBaseController
{

    /**
     * @apiDefine OfficialAccount 公众号
     */

    /**
     * @apiVersion 1.0.0
     * @api {GET} OfficialAccount/officialMenusList 获取自动配置的菜单
     * @apiGroup OfficialAccount
     * @apiName officialMenusList
     * @apiSuccess {String}  type 菜单类型.
     * @apiSuccess {String}  name 菜单名称.
     * @apiSuccess {String}  url 链接.
     * @apiSuccess {String}  name 名称.
     * @apiSuccess {String}  sub_button 子菜单.
     * @apiSuccessExample {json} Success-Response:·
     *
     *      {
     *          "code": 0,
     *          "msg": "",
     *          "data": [
     *              {
     *                  "type": "view",
     *                  "name": "继续阅读",
     *                  "url": "www.baidu.comcontinue"
     *              },
     *              {
     *                  "type": "view",
     *                  "name": "书城首页",
     *                  "url": "www.baidu.com"
     *              },
     *              {
     *                  "name": "用户中心",
     *                  "sub_button": [
     *                      {
     *                          "type": "view",
     *                          "name": "个人中心",
     *                          "url": "www.baidu.comuser"
     *                      },
     *                      {
     *                          "type": "view",
     *                          "name": "阅读记录",
     *                          "url": "www.baidu.combookshelf"
     *                      },
     *                      {
     *                          "type": "view",
     *                          "name": "我要充值",
     *                          "url": "www.baidu.compaycenter"
     *                      },
     *                      {
     *                          "type": "click",
     *                          "name": "联系客服",
     *                          "key": "contact_customer"
     *                      }
     *                  ]
     *              }
     *          ]
     *      }
     */

    function officialMenusList(Request $request)
    {

        $appid = $request->has('appid') ? $request->input('appid') : '';

        if (empty($appid)) {
            return response()->error("PARAM_EMPTY");
        }

        $officialAccount = OfficialAccount::officialAccountByAppid($appid);

        if (!empty($officialAccount)) {

            $distribution_channel_id = $officialAccount['distribution_channel_id'];
            $host = env('WECHAT_CUSTOM_HOST');
            
            if(in_array($distribution_channel_id,array(5,123))){
//             	$host = 'imycmh';
            }
            
//             $base_url = env('PROTOCOL') . '://site' . $distribution_channel_id . '.' . $host . '.com/';
//             // 渠道id加密
//             if(in_array($distribution_channel_id,array(123))){
//             	$encodeDistributionChannelId = encodeDistributionChannelId($distribution_channel_id);
//             	$base_url = env('PROTOCOL') . '://site' . $encodeDistributionChannelId . '.' . $host . '.com/';
//             }
            
            // 获取渠道配置，如果是云栖迁移站点的
            $channel = Channel::getById($distribution_channel_id);
            $is_yq_move = isset($channel->is_yq_move)?$channel->is_yq_move:'0';
            $is_domain_simple = isset($channel->is_domain_simple)?$channel->is_domain_simple:'0';
            if($is_domain_simple == '1'){
            	$encodeDistributionChannelId = $distribution_channel_id;
            }else{
            	$encodeDistributionChannelId = encodeDistributionChannelId($distribution_channel_id);
            }
            \Log::info('officialMenusList：distribution_channel_id:'.$distribution_channel_id.' encodeDistributionChannelId:'.$encodeDistributionChannelId.' is_yq_move:'.$is_yq_move.' is_domain_simple:'.$is_domain_simple);
            
            $base_url = env('PROTOCOL') . '://site' . $encodeDistributionChannelId . '.' . $host . '.com/';
            $help_url = 'https://help.zhuishuyun.com?distribution_channel_id='.$encodeDistributionChannelId;

            if($is_yq_move){
            	\Log::info('officialMenusList_is_yunqi_move:'.$distribution_channel_id);
            	
            	$buttons =
            	[
	            	[
		            	"type" => "click",
		            	"name" => "最近阅读",
		            	"key" => "recent_read", 
	            	],
            	];
            	
            }else{
            	\Log::info('officialMenusList_not_yunqi_move:'.$distribution_channel_id);
            	
            	$buttons =
            	[
	            	[
		            	"type" => "click",
		            	"name" => "最近阅读",
		            	"key" => "recent_read", 
	            	],
	            	[
		            	"type" => "click",
		            	"name" => "今日签到",
		            	"key" => 'daily_sign',
	            	],
	            	[
		            	"name" => "用户中心",
		            	"sub_button" =>
		            	[
			            	[
			            	"type" => "view",
			            	"name" => "个人中心",
			            	"url" => $base_url . 'person',
			            	],
			            	[
			            	"type" => "view",
			            	"name" => "书城首页",
			            	"url" => $base_url,
			            	],
			            	[
			            	"type" => "view",
			            	"name" => "优惠充值",
			            	"url" => $base_url . 'pay',
			            	],
			            	[
			            	"type" => "view",
			            	"name" => "帮助中心",
			            	"url" => $help_url,
			            	],
	            	    ],
	            	],
            	];
            	
            }
            

            return response()->success($buttons);
        }
    }


}
